@@ -6,6 +6,29 @@ class TestBlocksHTML(util.MdCase):
     """Test Blocks HTML cases."""
 
     extension = ['pymdownx.blocks.html', 'md_in_html']
+    extension_configs = {
+        'pymdownx.blocks.html': {
+            'custom': [
+                {'tag': 'custom', 'mode': 'block'}
+            ]
+        }
+    }
+
+
+    def test_raw_empty_block(self):
+        """Test that raw empty blocks are handled properly."""
+
+        self.check_markdown(
+            R'''
+            /// html | pre
+            ///
+            ''',
+            R'''
+            <pre></pre>
+            ''',
+            True
+        )
+
 
     def test_bad_tag(self):
         """Test bad HTML tag."""
@@ -20,7 +43,7 @@ class TestBlocksHTML(util.MdCase):
             <p>/// html | 3tag
             Some <em>content</em>
             ///</p>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -37,7 +60,7 @@ class TestBlocksHTML(util.MdCase):
             <p>/// html
             Some <em>content</em>
             ///</p>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -57,7 +80,7 @@ class TestBlocksHTML(util.MdCase):
             <p>Some <em>content</em></p>
             <p>And more <code>content</code>.</p>
             </div>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -76,7 +99,7 @@ class TestBlocksHTML(util.MdCase):
             <span>Will be parsed as inline <em>content</em>
 
             And more <code>content</code>.</span>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -95,7 +118,7 @@ class TestBlocksHTML(util.MdCase):
             <pre>Some *content*
 
             And more `content`.</pre>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -116,7 +139,7 @@ class TestBlocksHTML(util.MdCase):
             <div>Some *content*
 
             And more `content`.</div>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -137,7 +160,7 @@ class TestBlocksHTML(util.MdCase):
             <div>Will be parsed as inline <em>content</em>
 
             And more <code>content</code>.</div>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -156,7 +179,7 @@ class TestBlocksHTML(util.MdCase):
             ''',
             r'''
             <span><p>Some <em>content</em></p><p>And more <code>content</code>.</p></span>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -176,7 +199,7 @@ class TestBlocksHTML(util.MdCase):
             <p>Some <em>content</em></p>
             <p>And more <code>content</code>.</p>
             </div>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -251,8 +274,6 @@ class TestBlocksHTML(util.MdCase):
             **content**
             </div>
 
-
-
             <strong>content</strong></div>
             ''',
             True
@@ -309,7 +330,7 @@ class TestBlocksHTML(util.MdCase):
         )
 
     def test_html_and_script(self):
-        """Test inline format that script."""
+        """Test inline format with script."""
 
         self.check_markdown(
             R'''
@@ -325,3 +346,38 @@ class TestBlocksHTML(util.MdCase):
             ''',
             True
         )
+
+    def test_custom(self):
+        """Test custom block handling."""
+
+        self.check_markdown(
+            R'''
+            /// html | custom
+            - a
+            - b
+            ///
+            ''',
+            '''
+            <custom><ul><li>a</li><li>b</li></ul></custom>
+            ''',
+            True
+        )
+
+    def test_custom_override(self):
+        """Test custom block handling but mode is overridden."""
+
+        self.check_markdown(
+            R'''
+            /// html | custom
+                markdown: inline
+
+            - a
+            - b
+            ///
+            ''',
+            '''
+            <custom>- a
+            - b</custom>
+            ''',
+            True
+        )
