@@ -23,10 +23,10 @@ class TestHighlightGuess(util.MdCase):
             ```
             ''',
             '''
-            <div class="highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
-            ''',
+            ''',  # noqa: E501
             True
         )
 
@@ -52,10 +52,10 @@ class TestHighlightGuessBlock(util.MdCase):
             ```
             ''',
             '''
-            <div class="highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
-            ''',
+            ''',  # noqa: E501
             True
         )
 
@@ -92,7 +92,7 @@ class TestHighlightAutoTitle(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><span class="filename">Python Console Session</span><pre><span></span><code><span class="gp">&gt;&gt;&gt; </span><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><span class="filename">Python Console Session</span><pre><span></span><code><span class="gp">&gt;&gt;&gt; </span><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             </code></pre></div>
             ''',  # noqa: E501
             True
@@ -122,7 +122,7 @@ class TestHighlightAutoTitleMap(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><span class="filename">Python</span><pre><span></span><code><span class="gp">&gt;&gt;&gt; </span><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><span class="filename">Python</span><pre><span></span><code><span class="gp">&gt;&gt;&gt; </span><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             </code></pre></div>
             ''',  # noqa: E501
             True
@@ -150,7 +150,7 @@ class TestHighlightInline(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><pre><span></span><code><span class="linenos" data-linenos="1 "></span><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="linenos" data-linenos="1 "></span><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="linenos" data-linenos="2 "></span><span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -179,7 +179,7 @@ class TestNoClass(util.MdCase):
             ```
             ''',
             r'''
-            <div><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -197,7 +197,7 @@ class TestNoClass(util.MdCase):
             ```
             ''',
             r'''
-            <div class="more"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="more"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -216,7 +216,7 @@ class TestNoClass(util.MdCase):
             ''',
             r'''
             <div class="more"><table class="table"><tr><td class="linenos"><div class="linenodiv"><pre><span></span><span class="normal">1</span>
-            <span class="normal">2</span></pre></div></td><td class="code"><div><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <span class="normal">2</span></pre></div></td><td class="code"><div><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div></td></tr></table></div>
             ''',  # noqa: E501
@@ -248,7 +248,7 @@ class TestNoClassNoPygments(util.MdCase):
             r'''
             <pre><code class="language-python">import test
             test.test()</code></pre>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -277,7 +277,7 @@ class TestCustomLangPrefixNoPygments(util.MdCase):
             r'''
             <pre class="highlight"><code class="lang-python">import test
             test.test()</code></pre>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -335,6 +335,28 @@ class TestNoPygments(util.MdCase):
             True
         )
 
+    def test_no_pygments_extraneous_new_lines(self):
+        """Test that extraneous new lines are not included."""
+
+        self.check_markdown(
+            r'''
+            Test
+
+                import test
+                test.test()
+
+
+            Test
+            ''',
+            r'''
+            <p>Test</p>
+            <pre class="highlight"><code>import test
+            test.test()</code></pre>
+            <p>Test</p>
+            ''',
+            True
+        )
+
 
 class TestNoPygmentsCustomLineClass(util.MdCase):
     """Test no Pygments with custom line number class."""
@@ -363,8 +385,7 @@ class TestNoPygmentsCustomLineClass(util.MdCase):
             r'''
             <p>Text</p>
             <pre class="highlight"><code>import test
-            test.test()
-            </code></pre>
+            test.test()</code></pre>
             <p>Text</p>
             ''',
             True
@@ -410,7 +431,7 @@ class TestHighlightSpecial(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><pre><span></span><code><span class="linenos" data-linenos="1 "></span><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="linenos" data-linenos="1 "></span><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="linenos special" data-linenos="2 "></span><span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -428,7 +449,7 @@ class TestHighlightSpecial(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><pre><span></span><code><span class="linenos special" data-linenos="1 "></span><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="linenos special" data-linenos="1 "></span><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="linenos special" data-linenos="2 "></span><span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -457,7 +478,7 @@ class TestDisabledLinenums(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -490,7 +511,7 @@ class TestDisabledLinenumsNoPygments(util.MdCase):
             <p><code>python linenums="1"
             import test
             test.test()</code></p>
-            ''',  # noqa: E501
+            ''',
             True
         )
 
@@ -517,7 +538,7 @@ class TestGlobalLinenums(util.MdCase):
             ''',
             r'''
             <div class="highlight"><table class="highlighttable"><tr><td class="linenos"><div class="linenodiv"><pre><span></span><span class="normal">1</span>
-            <span class="normal">2</span></pre></div></td><td class="code"><div><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <span class="normal">2</span></pre></div></td><td class="code"><div><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div></td></tr></table></div>
             ''',  # noqa: E501
@@ -535,7 +556,7 @@ class TestGlobalLinenums(util.MdCase):
             ```
             ''',
             r'''
-            <div class="highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
             </code></pre></div>
             ''',  # noqa: E501
@@ -563,7 +584,7 @@ class TestPygmentsLangClass(util.MdCase):
             ```
             ''',
             r'''
-            <div class="language-python highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <div class="language-python highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">test</span>
             </code></pre></div>
             ''',  # noqa: E501
             True
@@ -577,7 +598,7 @@ class TestPygmentsLangClass(util.MdCase):
             `#!python import test`
             ''',
             '''
-            <p><code class="language-python highlight"><span class="kn">import</span> <span class="nn">test</span></code></p>
+            <p><code class="language-python highlight"><span class="kn">import</span><span class="w"> </span><span class="nn">test</span></code></p>
             ''',  # noqa: E501
             True
         )
@@ -639,3 +660,71 @@ class TestExtendedLang(util.MdCase):
             ''',  # noqa: E501
             True
         )
+
+
+class TestDefaultLang(util.MdCase):
+    """Test default block language cases."""
+
+    extension = ['pymdownx.highlight', 'pymdownx.superfences', 'pymdownx.inlinehilite']
+    extension_configs = {
+        'pymdownx.highlight': {
+            'default_lang': 'python'
+        }
+    }
+
+    def test_default_block(self):
+        """Test that default language affects block, but not inline code."""
+
+        self.check_markdown(
+            '''
+            `import code`
+
+                import code
+
+            ```
+            import code
+            ```
+            ''',
+            '''
+            <p><code>import code</code></p>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">code</span>
+            </code></pre></div>
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span><span class="w"> </span><span class="nn">code</span>
+            </code></pre></div>
+            ''',  # noqa: E501
+            True
+        )
+
+
+class TestDefaultLangInline(util.MdCase):
+    """Test default inline language cases."""
+
+    extension = ['pymdownx.highlight', 'pymdownx.superfences', 'pymdownx.inlinehilite']
+    extension_configs = {
+        'pymdownx.inlinehilite': {
+            'style_plain_text': 'python'
+        }
+    }
+
+    def test_default_inline(self):
+        """Test that default language affects block, but not inline code."""
+
+        self.check_markdown(
+            '''
+            `import code`
+
+                import code
+
+            ```
+            import code
+            ```
+            ''',
+            '''
+            <p><code class="highlight"><span class="kn">import</span><span class="w"> </span><span class="nn">code</span></code></p>
+            <div class="highlight"><pre><span></span><code>import code
+            </code></pre></div>
+            <div class="highlight"><pre><span></span><code>import code
+            </code></pre></div>
+            ''',  # noqa: E501
+            True
+        )
