@@ -3,6 +3,7 @@ from .. import util
 import os
 from pymdownx.snippets import SnippetMissingError
 from unittest.mock import patch, MagicMock
+import urllib.error
 
 BASE = os.path.abspath(os.path.dirname(__file__))
 
@@ -29,7 +30,7 @@ class TestSnippetDedent(util.MdCase):
             ```text
             ---8<--- "indented.txt:py-section"
             ```
-            ''',  # noqa: W291
+            ''',
             R'''
             <div class="highlight"><pre><span></span><code>def some_method(self, param):
                 &quot;&quot;&quot;Docstring.&quot;&quot;&quot;
@@ -48,7 +49,7 @@ class TestSnippetDedent(util.MdCase):
             ```text
             ---8<--- "indented.txt:5:8"
             ```
-            ''',  # noqa: W291
+            ''',
             R'''
             <div class="highlight"><pre><span></span><code>def some_method(self, param):
                 &quot;&quot;&quot;Docstring.&quot;&quot;&quot;
@@ -67,7 +68,7 @@ class TestSnippetDedent(util.MdCase):
             Paragraph
 
                 ---8<--- "indented.txt:py-section"
-            ''',  # noqa: W291
+            ''',
             R'''
             <p>Paragraph</p>
             <pre><code>def some_method(self, param):
@@ -106,11 +107,11 @@ class TestSnippets(util.MdCase):
             - Testing indentation
 
                 ---8<--- "b.txt"
-            ''',  # noqa: W291
+            ''',
             R'''
-            <p>Snippet
-            Snippet
-            ---8&lt;--- "b.txt"</p>
+            <p>Snippet</p>
+            <p>Snippet</p>
+            <p>---8&lt;--- "b.txt"</p>
             <ul>
             <li>
             <p>Testing indentation</p>
@@ -151,7 +152,7 @@ class TestSnippets(util.MdCase):
                 # Un-nested Inline
                 --8<-- "a.txt"
                 --8<-- "; b.txt"
-            ''',  # noqa: W291
+            ''',
             R'''
             <p>Snippet</p>
             <p>Snippet</p>
@@ -166,7 +167,7 @@ class TestSnippets(util.MdCase):
             <p>Snippet</p>
             </li>
             </ul>
-            ''',  # noqa: W291
+            ''',
             True
         )
 
@@ -209,6 +210,158 @@ class TestSnippets(util.MdCase):
             True
         )
 
+    def test_start_1_1(self):
+        """Test beginning of file, single line range."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:1"
+            ''',
+            '''
+            <p>This is a multi-line</p>
+            ''',
+            True
+        )
+
+    def test_start_1_2(self):
+        """Test beginning of file span of multiple lines."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2"
+            ''',
+            '''
+            <p>This is a multi-line
+            snippet.</p>
+            ''',
+            True
+        )
+
+    def test_start_multi(self):
+        """Test multiple line specifiers."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2,8:9"
+            ''',
+            '''
+            <p>This is a multi-line
+            snippet.
+            This is the end of the file.
+            There is no more.</p>
+            ''',
+            True
+        )
+
+    def test_start_multi_no_end(self):
+        """Test multiple line specifiers but the last has no end."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2,8"
+            ''',
+            '''
+            <p>This is a multi-line
+            snippet.
+            This is the end of the file.
+            There is no more.</p>
+            ''',
+            True
+        )
+
+    def test_start_multi_no_start(self):
+        """Test multiple line specifiers but the last has no start."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2,:8"
+            ''',
+            '''
+            <p>This is a multi-line
+            snippet.
+            This is a multi-line
+            snippet.</p>
+            <p>Content resides on various lines.
+            If we use line specifiers,
+            we can select any number of lines we want.</p>
+            <p>This is the end of the file.</p>
+            ''',
+            True
+        )
+
+    def test_start_multi_hanging_comma(self):
+        """Test multiple line specifiers but there is a hanging comma."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2,"
+            ''',
+            '''
+            ''',
+            True
+        )
+
+    def test_negative_range(self):
+        """Test negative indexing range."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:-3:-2"
+            ''',
+            '''
+            <p>This is the end of the file.
+            There is no more.</p>
+            ''',
+            True
+        )
+
+    def test_negative_single(self):
+        """Test negative indexing single line."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:-2:-2"
+            ''',
+            '''
+            <p>There is no more.</p>
+            ''',
+            True
+        )
+
+    def test_mixed_negative(self):
+        """Test negative indexing single line."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:8:-2"
+
+            ---8<--- "lines.txt:-3:9"
+            ''',
+            '''
+            <p>This is the end of the file.
+            There is no more.</p>
+            <p>This is the end of the file.
+            There is no more.</p>
+            ''',
+            True
+        )
+
+    def test_start_negative_multi(self):
+        """Test multiple line specifiers with negative indexes."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "lines.txt:1:2,-3:-2"
+            ''',
+            '''
+            <p>This is a multi-line
+            snippet.
+            This is the end of the file.
+            There is no more.</p>
+            ''',
+            True
+        )
+
     def test_end_line_inline(self):
         """Test ending line with inline syntax."""
 
@@ -374,6 +527,41 @@ class TestSnippets(util.MdCase):
             True
         )
 
+    def test_whole_file_with_sections(self):
+        """Test whole file with sections."""
+
+        self.check_markdown(
+            R'''
+            ```
+            -8<- "section_nested.txt"
+            ```
+            ''',
+            '''
+            <div class="highlight"><pre><span></span><code>div {
+                color: red;
+                background-color: white;
+                padding: 16px
+            }
+
+
+            div {
+                color: red;
+                /* --8&lt;-- [start: css-section4] */
+                background-color: white;
+                padding: 16px
+                /* --8&lt;-- [end: css-section4] */
+            }
+
+            div {
+                color: red;
+                background-color: white;
+                padding: 16px
+            }
+            </code></pre></div>
+            ''',
+            True
+        )
+
     def test_section_ignore_double_start_section(self):
         """Test nested sections."""
 
@@ -454,6 +642,39 @@ class TestSnippets(util.MdCase):
         )
 
 
+class _PathLikeExampleObject:
+    def __fspath__(self):
+        return os.path.join(BASE, '_snippets')
+
+
+class TestSnippetsPathLike(util.MdCase):
+    """Test snippet cases with path-like objects."""
+
+    extension = [
+        'pymdownx.snippets'
+    ]
+
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': _PathLikeExampleObject()
+        }
+    }
+
+    def test_inline(self):
+        """Test inline."""
+
+        self.check_markdown(
+            R'''
+            ---8<--- "a.txt"
+            ''',
+            R'''
+            <p>Snippet</p>
+            ''',
+            True
+        )
+
+
+
 class TestSnippetsFile(util.MdCase):
     """Test snippet file case."""
 
@@ -481,6 +702,91 @@ class TestSnippetsFile(util.MdCase):
         )
 
 
+class TestSnippetsNested(util.MdCase):
+    """Test nested restriction."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': os.path.join(BASE, '_snippets', 'nested'),
+            'check_paths': True
+        }
+    }
+
+    def test_restricted(self):
+        """Test file restriction."""
+
+        with self.assertRaises(SnippetMissingError):
+            self.check_markdown(
+                R'''
+                --8<-- "../b.txt"
+                ''',
+                '''
+                <p>Snippet</p>
+                ''',
+                True
+            )
+
+
+class TestSnippetsMultiNested(util.MdCase):
+    """Test multi-nested restriction."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': os.path.join(BASE),
+            'check_paths': True
+        }
+    }
+
+    def test_restricted_multi_nested(self):
+        """Test file restriction."""
+
+        self.check_markdown(
+            R'''
+            --8<-- "_snippets/b.txt"
+            ''',
+            '''
+            <p>Snippet</p>
+            ''',
+            True
+        )
+
+
+class TestSnippetsNestedUnrestricted(util.MdCase):
+    """Test nested no bounds."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': os.path.join(BASE, '_snippets', 'nested'),
+            'restrict_base_path': False
+        }
+    }
+
+    def test_restricted(self):
+        """Test file restriction."""
+
+        self.check_markdown(
+            R'''
+            --8<-- "../b.txt"
+            ''',
+            '''
+            <p>Snippet</p>
+            ''',
+            True
+        )
+
+
 class TestSnippetsAutoAppend(util.MdCase):
     """Test snippet file case."""
 
@@ -661,7 +967,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = [b'contents']
+        cm.read.return_value = b'contents'
         cm.headers = {'content-length': '8'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -683,7 +989,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.side_effect = [[b'content', b'', b'--8<-- "https://test.com/myfile2.md"'], [b'other']]
+        cm.read.side_effect = [b'content\n\n--8<-- "https://test.com/myfile2.md"', b'other']
         cm.headers = {'content-length': '8'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -706,7 +1012,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.side_effect = [[b'content', b'', b'--8<-- "https://test.com/myfile.md"'], [b'other']]
+        cm.read.side_effect = [b'content\n\n--8<-- "https://test.com/myfile.md"', b'other']
         cm.headers = {'content-length': '8'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -728,7 +1034,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = [b'content', b'', b'--8<-- "b.txt"']
+        cm.read.return_value = b'content\n\n--8<-- "b.txt"'
         cm.headers = {'content-length': '8'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -747,17 +1053,14 @@ class TestURLSnippets(util.MdCase):
     def test_url_lines(self, mock_urlopen):
         """Test specifying specific lines in a URL."""
 
-        content = []
-        length = 0
         with open('tests/test_extensions/_snippets/lines.txt', 'rb') as f:
-            for l in f:
-                length += len(l)
-                content.append(l)
+            content = f.read()
+        length = len(content)
 
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = content
+        cm.read.return_value = content
         cm.headers = {'content-length': length}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -781,7 +1084,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 404
         cm.code = 404
-        cm.readlines.return_value = []
+        cm.read.return_value = b''
         cm.headers = {'content-length': '0'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -801,20 +1104,19 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = []
+        cm.read.return_value = b''
         cm.headers = {}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
 
-        with self.assertRaises(ValueError):
-            self.check_markdown(
-                R'''
-                --8<-- "https://test.com/myfile.md"
-                ''',
-                '''
-                ''',
-                True
-            )
+        self.check_markdown(
+            R'''
+            --8<-- "https://test.com/myfile.md"
+            ''',
+            '''
+            ''',
+            True
+        )
 
     @patch('urllib.request.urlopen')
     def test_missing_content_length_too_big(self, mock_urlopen):
@@ -823,7 +1125,7 @@ class TestURLSnippets(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = []
+        cm.read.return_value = b''
         cm.headers = {'content-length': str(1024 * 1024 * 48)}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -839,13 +1141,33 @@ class TestURLSnippets(util.MdCase):
             )
 
     @patch('urllib.request.urlopen')
+    def test_content_too_big_no_content_length(self, mock_urlopen):
+        """Test content length too big."""
+
+        cm = MagicMock()
+        cm.status = 200
+        cm.code = 200
+        cm.read = lambda count: b"-" * (1024 * 1024 * 48)
+        cm.__enter__.return_value = cm
+        mock_urlopen.return_value = cm
+
+        with self.assertRaises(ValueError):
+            self.check_markdown(
+                R'''
+                --8<-- "https://test.com/myfile.md"
+                ''',
+                '''
+                ''',
+                True
+            )
+    @patch('urllib.request.urlopen')
     def test_content_length_zero(self, mock_urlopen):
         """Test empty content."""
 
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = []
+        cm.read.return_value = b''
         cm.headers = {'content-length': '0'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -862,17 +1184,14 @@ class TestURLSnippets(util.MdCase):
     def test_url_sections(self, mock_urlopen):
         """Test specifying a section in a URL."""
 
-        content = []
-        length = 0
         with open('tests/test_extensions/_snippets/section.txt', 'rb') as f:
-            for l in f:
-                length += len(l)
-                content.append(l)
+            content = f.read()
+        length = len(content)
 
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = content
+        cm.read.return_value = content
         cm.headers = {'content-length': length}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -907,17 +1226,14 @@ class TestURLDedentSnippets(util.MdCase):
     def test_url_sections(self, mock_urlopen):
         """Test specifying a section in a URL."""
 
-        content = []
-        length = 0
         with open('tests/test_extensions/_snippets/indented.txt', 'rb') as f:
-            for l in f:
-                length += len(l)
-                content.append(l)
+            content = f.read()
+        length = len(content)
 
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = content
+        cm.read.return_value = content
         cm.headers = {'content-length': length}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -961,7 +1277,7 @@ class TestURLSnippetsNoMax(util.MdCase):
         cm = MagicMock()
         cm.status = 200
         cm.code = 200
-        cm.readlines.return_value = [b'contents']
+        cm.read.return_value = b'contents'
         cm.headers = {'content-length': str(1024 * 1024 * 48)}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -1000,7 +1316,7 @@ class TestURLSnippetsMissing(util.MdCase):
         cm = MagicMock()
         cm.status = 404
         cm.code = 404
-        cm.readlines.return_value = []
+        cm.read.return_value = b''
         cm.headers = {'content-length': '0'}
         cm.__enter__.return_value = cm
         mock_urlopen.return_value = cm
@@ -1013,3 +1329,159 @@ class TestURLSnippetsMissing(util.MdCase):
                 '',
                 True
             )
+
+
+class TestURLSnippetsRetry(util.MdCase):
+    """Test snippet URL retry cases."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': [os.path.join(BASE, '_snippets')],
+            'url_download': True,
+            'check_paths': True
+        }
+    }
+
+    @patch('urllib.request.urlopen')
+    @patch('time.sleep')
+    def test_successful_retry(self, mock_sleep, mock_urlopen):
+        """Test successful retry after 429."""
+
+        cm = MagicMock()
+        cm.status = 200
+        cm.code = 200
+        cm.read.return_value = b'content\n'
+        cm.headers = {'content-length': '8'}
+        cm.__enter__.return_value = cm
+        cm.__exit__.return_value = None  # Mock __exit__
+
+        mock_urlopen.side_effect = [
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            cm
+        ]
+
+        self.check_markdown(
+            R'''
+            --8<-- "http://example.com"
+            ''',
+            '''
+            <p>content</p>
+            ''',
+            True
+        )
+
+        mock_sleep.assert_any_call(2)
+        mock_sleep.assert_any_call(4)
+
+    @patch('urllib.request.urlopen')
+    @patch('time.sleep')
+    def test_exhaust_retries(self, mock_sleep, mock_urlopen):
+        """Test exhausting all retries."""
+
+        mock_urlopen.side_effect = [
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+        ]
+
+        with self.assertRaises(SnippetMissingError) as cm:
+            self.md.convert(
+                R'''
+                --8<-- "http://example.com"
+                '''
+            )
+
+        self.assertIn("Cannot download snippet 'http://example.com' (HTTP Error 429)", str(cm.exception))
+
+        mock_sleep.assert_any_call(2)
+        mock_sleep.assert_any_call(4)
+        mock_sleep.assert_any_call(6)
+
+
+class TestURLSnippetsRetryCustomBackoff(util.MdCase):
+    """Test snippet URL retry cases with custom backoff."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': [os.path.join(BASE, '_snippets')],
+            'url_download': True,
+            'backoff_factor': 1.5,
+            'check_paths': True
+        }
+    }
+
+    @patch('urllib.request.urlopen')
+    @patch('time.sleep')
+    def test_custom_backoff(self, mock_sleep, mock_urlopen):
+        """Test custom backoff factor."""
+
+        cm = MagicMock()
+        cm.status = 200
+        cm.code = 200
+        cm.read.return_value = b'content\n'
+        cm.headers = {'content-length': '8'}
+        cm.__enter__.return_value = cm
+        cm.__exit__.return_value = None  # Mock __exit__
+
+        mock_urlopen.side_effect = [
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            cm
+        ]
+
+        self.check_markdown(
+            R'''
+            --8<-- "http://example.com"
+            ''',
+            '''
+            <p>content</p>
+            ''',
+            True
+        )
+
+        mock_sleep.assert_called_once_with(1.5)
+
+
+class TestURLSnippetsRetryCustomRetries(util.MdCase):
+    """Test snippet URL retry cases with custom retries."""
+
+    extension = [
+        'pymdownx.snippets',
+    ]
+    extension_configs = {
+        'pymdownx.snippets': {
+            'base_path': [os.path.join(BASE, '_snippets')],
+            'url_download': True,
+            'max_retries': 2,
+            'check_paths': True
+        }
+    }
+
+    @patch('urllib.request.urlopen')
+    @patch('time.sleep')
+    def test_custom_max_retries(self, mock_sleep, mock_urlopen):
+        """Test custom max retries."""
+
+        mock_urlopen.side_effect = [
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+            urllib.error.HTTPError('http://example.com', 429, 'Too Many Requests', {}, None),
+        ]
+
+        with self.assertRaises(SnippetMissingError) as cm:
+            self.md.convert(
+                R'''
+                --8<-- "http://example.com"
+                '''
+            )
+        self.assertIn("Cannot download snippet 'http://example.com' (HTTP Error 429)", str(cm.exception))
+
+        mock_sleep.assert_any_call(2)
+        mock_sleep.assert_any_call(4)
